import os
import argparse
import logging

from RAAspotter import RAAspotter
from RAAspotter_ppparis import RAAspotter_ppparis

# Config
__RAA_PAGE = 'https://www.prefecturedepolice.interieur.gouv.fr/actualites-et-presse/arretes/accueil-arretes'
__USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36'
__headless_mode = True
__KEYWORDS = os.getenv('KEYWORDS') or 'vidéoprotection,caméras,captation,aéronef'
__DATA_DIR = os.path.dirname(os.path.abspath(__file__))+'/data/ppparis/'

# Début du script
parser = argparse.ArgumentParser(prog='ppparis.py', description='Télécharge les RAA de la Préfecture de police de Paris et recherche des mots-clés')
parser.add_argument('-k', '--keywords', action='store', help='liste des termes recherchés, séparés par une virgule (par défaut : vidéoprotection,caméras,captation,aéronef)')
parser.add_argument('-v', action='store_true', help='relève le niveau de verbosité à INFO')
parser.add_argument('-vv', action='store_true', help='relève le niveau de verbosité à DEBUG')
args = parser.parse_args()

if args.v or os.getenv('VERBOSE'):
  logging.basicConfig(level=logging.INFO)

if args.vv or os.getenv('VVERBOSE'):
  logging.basicConfig(level=logging.DEBUG)

if args.keywords:
  __KEYWORDS = args.keywords

# On crée le dossier de téléchargement
os.makedirs(__DATA_DIR, exist_ok=True)

raa_spotter = RAAspotter_ppparis(__DATA_DIR, __USER_AGENT)

RAAspotter_ppparis.print_output('RAAspotter_ppparis')
RAAspotter_ppparis.print_output(f'Termes recherchés: {__KEYWORDS}')

page_content = raa_spotter.get_session(__RAA_PAGE, "block-decree-list-block")

raa_elements = raa_spotter.get_raa(page_content)

raa_spotter.parse_raa(raa_elements, __KEYWORDS.split(','))
